@@ -21,7 +21,7 @@ context = {
 ## render template
 import tenjin
 from tenjin.helpers import *
-from tenjin.helpers.html import text2html
+from tenjin.html import text2html
 engine = tenjin.Engine(path=['views'], layout='_layout.pyhtml')
 html = engine.render('blog-post.pyhtml', context)
 print(html)
