@@ -1,6 +1,6 @@
 import tenjin
 from tenjin.helpers import *
-from tenjin.helpers.html import text2html
+from tenjin.html import text2html
 engine = tenjin.Engine(path=['views'])
 context = {
     'title': 'Blog Post Test',
